/****************************************************************************
** $Id: qt/gltexobj.h   3.1.2   edited Nov 8 2002 $
**
** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
**
** This file is part of an example program for Qt.  This example
** program may be used, distributed and modified without limitation.
**
*****************************************************************************/

/****************************************************************************
**
** This is a simple QGLWidget displaying an openGL wireframe box
**
****************************************************************************/

#ifndef GLTEXOBJ_H
#define GLTEXOBJ_H

#include <QGLWidget>

#include "point.h"
#include "kdtree.h"

class GLTexobj : public QGLWidget
{
	Q_OBJECT

public:

	GLTexobj( QWidget* parent );
	~GLTexobj();

public slots:

	void			clearQuery();
	void			clearAll();
	void			growkdtree();
	void			setknnval(const QString& str);
	void			mousePressEvent(QMouseEvent* e);
	void			mouseReleaseEvent(QMouseEvent* e);
	void			mouseDoubleClickEvent(QMouseEvent* e);
	void			mouseMoveEvent(QMouseEvent* e);
        void			wheelEvent(QWheelEvent* e);
	void			mapCoords(int,int);
	void			update();
	void			fileOpen();
	void			fileSaveAs();
	void			imgSaveAs();
	void			about();

protected:

	void			initializeGL();
	void			paintGL();
	void			resizeGL(int w, int h);

private:
	QTimer*			timer;
	QImage			img;
	QFont			textfont;

	double			x,y;		// manually input point
	double			x1,y1,x2,y2;	// bounding box query
	bool			drawingBox;	// bounding box drawing flag

	std::vector<point_t* >	pts;		// the raw point list

	kdtree_t<point_t,point_t*,point_c>	kdtree;		// the kd-tree

	point_t*		query;		// the nn and k-nn query point
	point_t*		nearest;	// resulting nn point
	double			radius;		// radius of (k-)nn query

	int			k;		// k for k-nn query
	std::vector<point_t* >	knearest;	// resulting k-nn point list
	std::vector<point_t* >	range;		// resulting range point list
};


#endif // GLTEXOBJ_H
