/****************************************************************************
** $Id: qt/glwinobj.cpp   3.1.2   edited Nov 8 2002 $
**
** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
**
** This file is part of an example program for Qt.  This example
** program may be used, distributed and modified without limitation.
**
*****************************************************************************/

#include <iostream>
#include <vector>
#include <string>
#include <sstream>
#include <list>
#include <cmath>

#include <QApplication>
#include <QWidget>
#include <QMenuBar>
#include <QMenu>
#include <QVBoxLayout>

#include "glwinobj.h"
#include "gltexobj.h"

GLObjectWindow::GLObjectWindow( QWidget* parent, Qt::WindowFlags f ):
	QWidget( parent, f )
{
  // Create an OpenGL widget: (doubleBuffer | rgba | depth) set globally
  GLTexobj* texwin = new GLTexobj(this);
  std::cout << "doubleBuffer: " << texwin->format().doubleBuffer() << " "
            << "rgba: "         << texwin->format().rgba()         << " "
            << "depth: "        << texwin->format().depth()        << " "
            << std::endl;
  texwin->setMouseTracking(true);

  // create the file menu
  QMenu *file = new QMenu("File",this);
  file->addAction("Open...",texwin,SLOT(fileOpen()),Qt::CTRL+Qt::Key_O);
  file->addAction("Save As...",texwin,SLOT(fileSaveAs()),Qt::CTRL+Qt::Key_W);
  file->addSeparator();
  file->addAction("Quit",QApplication::instance(),SLOT(quit()),Qt::CTRL+Qt::Key_Q);

  // create the edit menu
  QMenu *edit = new QMenu("Edit",this);
  edit->addAction("Clear Query",texwin,SLOT(clearQuery()),Qt::CTRL+Qt::Key_C);
  edit->addAction("Clear All",texwin,SLOT(clearAll()),Qt::CTRL+Qt::Key_A);
  edit->addAction("KD-Tree",texwin,SLOT(growkdtree()),Qt::CTRL+Qt::Key_K);

  // create the image menu
  QMenu *image = new QMenu("Image",this);
  image->addAction("Save As...",texwin,SLOT(imgSaveAs()));

  // create the help menu
  QMenu *help = new QMenu("Help",this);
  help->addAction("Help",texwin,SLOT(about()),Qt::CTRL+Qt::Key_H);

  // create a menu bar
  QMenuBar *menubar = new QMenuBar(this);
  menubar->addSeparator();
  menubar->addMenu(file);
  menubar->addMenu(edit);
  menubar->addMenu(image);
  menubar->addSeparator();
  menubar->addMenu(help);

  // Top level layout (with 0,0 border)
  QVBoxLayout *vlayout = new QVBoxLayout(this);
  vlayout->setSpacing(0);
  vlayout->setMargin(0);
  vlayout->setMenuBar(menubar);
  vlayout->addWidget(texwin);
}
